#
#   This file is part of SuRDebug, Copyright (C) 2009 Pecan Heber
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
#   Copyright: (c) 2009 Pecan Heber
#
# ----------------------------------------------------------------------------
# Switch SketchUp Ruby lib Dll between versions 180 and 186
#
#   Usage: set|restore SketchUp_Folder Ruby_Folder
# ----------------------------------------------------------------------------
module LibSetRst

    require "fileutils"
    include FileUtils::Verbose

    operation = ARGV[0];
    sketchupDir = ARGV[1];
    rubyDir = ARGV[2];
    suPid = ARGV[3];
    libDll = "msvcrt-ruby18.dll"
    operation = "" if (operation == nil);
    sketchupDir = "" if (sketchupDir == nil);
    rubyDir = "" if (rubyDir == nil);
    if (suPid == nil): suPid = 0; end;
    if (suPid == ""): suPid = 0; end;
    suPid = suPid.to_i();

    printf("Input: Op[%s] SketchUp[%s] Ruby[%s]\n", operation, sketchupDir, rubyDir);
    if ( (operation != "set") && (operation != "restore") )
        puts "\nUsage: set|restore SketchUp_Folder Ruby_Folder\n"
        printf("Args 1:%s 2:%s 3:%s 4:%s\n", ARGV[0],ARGV[1],ARGV[2],ARGV[3]);
        exit();
    end;
    if (sketchupDir == "") || (not File::exists?(sketchupDir + "/" + libDll) )
        puts "\nERROR: Cannot find SketchUp lib: #{sketchupDir}"+"/"+libDll;
        exit(1);
    end;
    if (rubyDir == "") || (not File::exists?(rubyDir+"/bin"+"/"+libDll) )
        puts "\nERROR: Cannot find installed Ruby lib: " + rubyDir+"/bin"+"/"+libDll
        exit(1);
    end;

    ## ----------------------------------------------------
    ## # Testing environment setup
    ## ----------------------------------------------------
    ## treat temp dir as if it was sketchup dir
    ##fileSrc = sketchupDir +"/"+ libDll;
    ##fileTarget = "/temp" +"/"+ libDll;
    ##FileUtils.cp( fileSrc, fileTarget);
    ##sketchupDir = "/temp"; #**TESTING**

    if (RUBY_VERSION < "1.8.6")
        puts("Ruby/bin version #{RUBY_VERSION} not supported");
        exit();
    end;
    currentVer = RUBY_VERSION.gsub(/[.]/, ''); # '1.8.5' => '186'
    puts "Running ruby version is "+currentVer;

    rubyBin = rubyDir +"/bin";
    #kill SketchUp since it has locked the Ruby lib dll
    #FIXME: tskill is not cross platform
    if (suPid > 0): system("tskill #{suPid}"); end;
    system("tskill SuRDebug");
    system("tskill SketchUp");
    sleep(1);
    if (operation == "set")
        # ----------------------------------------------------
        # Copy Ruby/bin lib to SketchUp directory
        # ----------------------------------------------------
        # we should be  running out of the Ruby/bin lib
        fileSrc = rubyBin+ "/"+ libDll;
        fileTarget = sketchupDir +"/"+ libDll.gsub("18", currentVer);
        if ( not File::exists?(fileTarget) )
            puts "Copying " + fileSrc +" to "+ fileTarget;
            FileUtils.cp( fileSrc, fileTarget);
        else
            puts fileTarget + " already exists.";
        end;
        # ----------------------------------------------------
        # Backup SketchUp Ruby version dll
        # ----------------------------------------------------
        fileSrc = sketchupDir +"/"+ libDll;
        if (File::size(fileSrc) != 811008)
            printf("\nERROR: The current Sketchup lib does not appear to be 1.8.0\n");
            printf("Current lib File:%s size: %d\n", fileSrc, File::size(fileSrc));
            exit(1);
        end;
        fileTarget = sketchupDir +"/"+ libDll.gsub("18", "180");
        if ( not File::exists?(fileTarget) )
            puts "Copying " + fileSrc +" to "+ fileTarget;
            FileUtils.cp( fileSrc, fileTarget);
        else
            puts fileTarget + " already exists.";
        end;
        # ----------------------------------------------------
        # Clobber SketchUp dll with Ruby/bin dll as the current lib dll
        # ----------------------------------------------------
        fileSrc = sketchupDir +"/"+ libDll.gsub("18", "186");;
        fileTarget = sketchupDir +"/"+ libDll;
        if ( not (File::size(fileSrc) == File::size(fileTarget)) )
            puts "Copying " + fileSrc +" to "+ fileTarget;
            FileUtils.cp( fileSrc, fileTarget);
        else
            puts fileSrc +" already active as "+ fileTarget ;
        end;
    end;#set operation

    if (operation == "restore")
        # ----------------------------------------------------
        # Restore SketchUp dll with original SketchUp dll as the current lib dll
        # ----------------------------------------------------
        fileSrc = sketchupDir +"/"+ libDll.gsub("18", "180");;
        if (not File::exists?(fileSrc))
            puts("\nERROR: Did not find file to restore: "+ fileSrc);
            exit(1);
        end;
        fileTarget = sketchupDir +"/"+ libDll;
        if ( not (File::size(fileSrc) == File::size(fileTarget)) )
            puts "Restore " + fileSrc +" to "+ fileTarget;
            FileUtils.cp( fileSrc, fileTarget);
        else
            puts fileSrc +" already active as "+ fileTarget ;
        end;
    end;#restore operation

    cmd = "\"#{sketchupDir}/sketchup\""
    puts "Starting "+cmd;
    thr = Thread.new() do system(cmd); end;
end;#module LibSetRst
