
# Overridden code from Ruby-Debug

module Debugger
    class Interface # :nodoc:
        attr_writer :have_readline  # true if Readline is available

        def initialize
            @have_readline = false
        end

        # Common routine for reporting debugger error messages.
        # Derived classed may want to override this to capture output.
        def errmsg(*args)
            if Debugger.annotate.to_i > 2
                aprint 'error-begin'
                print(*args)
                aprint ''
            else
                print '*** '
                print(*args)
            end
        end

        # Format msg with gdb-style annotation header
        def afmt(msg, newline="\n")
            "\032\032#{msg}#{newline}"
            #"++#{msg}#{newline}"
        end

        def aprint(msg)
            print afmt(msg)
        end

    end;#class Interface

    class LocalInterface < Interface # :nodoc:
        attr_accessor :command_queue
        attr_accessor :histfile
        attr_accessor :history_save
        attr_accessor :history_length
        attr_accessor :restart_file

        unless defined?(FILE_HISTORY)
            FILE_HISTORY = ".rdebug_hist"
        end
        def initialize()
            super
            @command_queue = []
            @have_readline = false
            @history_save = true
            # take gdb's default
            @history_length = ENV["HISTSIZE"] ? ENV["HISTSIZE"].to_i : 256
            @histfile = File.join(ENV["HOME"]||ENV["HOMEPATH"]||".",
            FILE_HISTORY)
            open(@histfile, 'r') do |file|
                file.each do |line|
                    line.chomp!
                    #Readline::HISTORY << line //(pecan 2009/2/20)
                end
            end if File.exist?(@histfile)
            @restart_file = nil
        end

        def read_command(prompt)
            readline(prompt, true)
        end

        def confirm(prompt)
            ##readline(prompt, false) //(pecan 2009/3/16)
            'y'
        end

        def print(*args)
            #STDOUT.printf(*args)
            #@fh.printf(*args)
            $stdout.flush();
            $stdout.printf(*args);
            $stdout.flush();
        end

        def close
        end

        # Things to do before quitting
        def finalize
            print "finialize \n" #//(pecan 2009/3/14) never happens
            $stdout.flush()
            if Debugger.method_defined?("annotate") and Debugger.annotate.to_i > 2
                print "\032\032exited\n\n"
            end
            print "\032\032exited\n\n" #//(pecan 2009/4/06)
            if Debugger.respond_to?(:save_history)
                Debugger.save_history
            end
        end

        def readline_support?
            @have_readline
        end

        private
        begin
            require 'readline'
            class << Debugger
                @have_readline = true
                define_method(:save_history) do
                    iface = self.handler.interface
                    iface.histfile ||= File.join(ENV["HOME"]||ENV["HOMEPATH"]||".",
                    FILE_HISTORY)
                    open(iface.histfile, 'w') do |file|
                        Readline::HISTORY.to_a.last(iface.history_length).each do |line|
                            file.puts line unless line.strip.empty?
                        end if defined?(iface.history_save) and iface.history_save
                    end rescue nil
                end
                public :save_history
            end
            Debugger.debug_at_exit do
                print "finalize\n"  #//(pecan 2009/3/14) never happens
                puts  "finalize"    #//(pecan 2009/3/14)
                $stdout.flush
                finalize if respond_to?(:finalize)
            end

            def readline(prompt, hist)
                ##Readline::readline(prompt, hist)
                line = SuRDebug.GetDebugInput(prompt);
                line.chomp!
                line
            end
        rescue LoadError
            def readline(prompt, hist)
                @histfile = ''
                @hist_save = false
                #STDOUT.print prompt
                #STDOUT.flush
                #line = STDIN.gets
                #exit unless line
                line = SuRDebug.GetDebugInput(prompt);
                line.chomp!
                line
            end
        end;#begin/rescue
    end;#class LocalInterface



end;#module
