# -----------------------------------------------------------------------------
# Copyright 2009, Pecan Heber.

# Permission to use, copy, modify, and distribute this software for
# any purpose and without fee is hereby granted, provided that the above
# copyright notice appear in all copies.

# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
# -----------------------------------------------------------------------------
# File: SuRDebug.rb
#
# Use: Communicate between the Ruby-Debugger and SuRDebug.exe
#      Sets environment for debugger and pipes for I/O
#

#require "sketchup.rb"

# ---------------------------------------------
module SuRDebug # SketchUp Ruby Editor Debugger
# ---------------------------------------------

    puts "Debugger working from: #{Dir.pwd()}"
    @SuRDebugInText
    # -------------------
    # Restart SketchUp
    # -------------------
    def self.RestartSketchUp()
        KillSketchUp(true);
    end;#RestartSketchUp()
    def self.KillSketchUp(restart = false)
        if (defined?(Debugger))
            while Debugger.started?(): Debugger.stop(); end;
        end;
        pid = Process.pid();
        #kill current process from a thread
        thr = Thread.new(){
                while (true)
                    sleep(1);
                    Process.kill("KILL", pid);
                    system("tskill SuRDebug");
                end;
            }
        #start another process before we die
        #exec("sketchup"); 'exec' hangs the original process on Windows
        if (restart): system("sketchup"); end;
        #hack it until it's dead!
        while true: Thread.pass(); Process.kill("KILL", pid); end;
    end;#KillSketchUp

    # --------------------------------------------------------
    # Verify Ruby version/install needed dll
    # --------------------------------------------------------
    # SketchUp ships with Ruby 1.8.0 while debugger needs 1.8.4 or later
    # invoke .../ruby/bin/ruby to write lib version string to a temp file
    # Read version; check it; ask permission to install if needed.
    # Back up the current SketchUp ruby.dll and install updated dll.
    def self.VerifyRubyVersion()
        if ( RUBY_VERSION < "1.8.6" )
            rubyBin = "#{@rubyInstallPath}/bin"
            rubyBinLibVer = "unknown";
            cmd = "#{rubyBin}/ruby -e 'p RUBY_VERSION'";
            rubyBinLibVer = `#{cmd}`;
            rubyBinLibVer.gsub!( /[\"\n]/, '');
            puts "Ruby Version"+ rubyBinLibVer;
            msg = "SuRDebug requires Ruby version 1.8.6 . \n" <<
                   "Switch to installed Ruby lib version #{rubyBinLibVer} ?"
            puts msg
            answ = UI.messagebox( msg, MB_YESNO,"SuRDebug");
            #answ 6 == yes, 7 == no
            if (answ == 7): return(false); end;
            sketchupDir = Sketchup.find_support_file("");
            # Invoke script to install newer ruby.dll then die to release
            # the SketchUp lock on the running dll.
            cmd = "#{rubyBin}/ruby \"#{@surdebugDir}/scripts/libsetrst.rb\" set \"#{sketchupDir}\" \"#{@rubyInstallPath}\" \"#{Process.pid().to_s()}\""
            puts cmd
            thr = Thread.new(){`#{cmd}`}
            KillSketchUp();
        end;#if RUBY_VERSION
        return(true);
    end;#VerifyRubyVersion
    #----------------------------------
    # Restore SketchUp Ruby Lib
    #----------------------------------
    def self.RestoreSULib()
        if ( RUBY_VERSION != "1.8.0" )
            @surdebugDir = File.join(Sketchup.find_support_file("Plugins"),"SuRDebug");
            loop{
                @rubyInstallPath = Sketchup.read_default("SuRDebug",
                        "RubyInstallPath", "unknown.");
                if (not File.exists?(@rubyInstallPath) )
                    load @surdebugDir+"/SetRubyInstallPath.rb"
                else break;
                end;
            }#end loop
            rubyBin = "#{@rubyInstallPath}/bin"
            sketchupDir = Sketchup.find_support_file("");
            # Invoke script to restore dll then die to release
            # SketchUp lock on the running dll.
            cmd = "#{rubyBin}/ruby \"#{@surdebugDir}/scripts/libsetrst.rb\" restore \"#{sketchupDir}\" \"#{@rubyInstallPath}\" \"#{Process.pid().to_s()}\" "
            ##thr = Thread.new() do system("cmd /k "+cmd); end;
            thr = Thread.new() {`#{cmd}`}
            KillSketchUp();
        end;#if RUBY_VERSION
        UI.messagebox("Current SU Lib already #{RUBY_VERSION} ");
        return(true);
    end;#RestoreSULib

    # --------------------------------------------------------
    # GetDebugInput
    # glue routine between debugger input and SuRDebug.exe
    # --------------------------------------------------------
    def self.GetDebugInput(prompt)
        # get debugger command from user
        if ((prompt != nil) or (prompt != "")): print(prompt); end;
        $stdout.flush();
        $stderr.flush();
        @SuRDebugInText = $stdin.gets();
        # remove line ending
        @SuRDebugInText.chomp!();
        @quit = true if (@SuRDebugInText == "quit");
        printf("Ruby<:[%s]\n",@SuRDebugInText); #debugging
        $stdout.flush()
        return @SuRDebugInText;
    end#GetUserInput
    def self.gets()
        input = GetDebugInput("");
        return input.chomp();
    end;
    # -------------------
    # IsQuit
    # -------------------
    def self.IsQuit()
        return @quit;
    end;

    # ----------------------------------------------------------
    # Run SuRDebug
    # ----------------------------------------------------------
    def self.RunSuRDebug()
      begin

        @quit = false;
        @rubyInstallPath = "";
        @surdebugDir = Dir.pwd; #Set in ::DebuggerStart()

        @savedStdout = $stdout;
        @savedStderr = $stderr;
        @savedStdin = $stdin;

        $stdout = STDOUT;
        $stderr = STDERR;
        $stdin = STDIN;

        # --------------------------------------------------------
        # Setup environment to run debugger and load users script
        # --------------------------------------------------------
        Sketchup.send_action("showRubyPanel:");
        puts("Current folder is: #{@surdebugDir}");
        # Find SuRDebug script folder
        sketchupDir =
        @surdebugDir = File.join(Sketchup.find_support_file("Plugins"),"SuRDebug")
        modifiedRubyDebugger = File.join(@surdebugDir, "/scripts/rdebugmod.rb")

        # Get Ruby install path
        loop{
            @rubyInstallPath = Sketchup.read_default("SuRDebug",
                    "RubyInstallPath", "unknown.");
            if (not File.exists?(@rubyInstallPath) )
                load @surdebugDir+"/scripts/SetRubyInstallPath.rb"
            else break;
            end;
        }#end loop

        modifiedRubyDebugger = @surdebugDir + "/scripts/rdebugmod.rb"

        # Add Ruby and SuRDebug scripts to lib paths
        rubyLibs = [@surdebugDir + "/scripts",
                    @surdebugDir + "/ruby-debug",
                    @surdebugDir + "/ruby-debug/ruby-debug",
                    @surdebugDir + "/ruby-debug/ruby-debug-base-0.10.3-mswin32/lib",
                    @surdebugDir + "/ruby-debug/linecache-0.43-mswin32/lib",
                    @surdebugDir + "/ruby-debug/columnize-0.3.0/lib",
                    @rubyInstallPath + "/lib/ruby/1.8",
                    @rubyInstallPath + "/lib/ruby/1.8/i386-mswin32" ];
        rubyLibs.each{|x| $:.push(x) unless $:.include?(x);}

        # Verify Ruby version/ install ruby dll if user permits
        if (not VerifyRubyVersion() )
            puts("SuRDebug cannot function with Ruby version "+ RUBY_VERSION);
            UI.messagebox("SuRDebug cannot function with Ruby version "+ RUBY_VERSION, MB_OK);
            return false;
        end;

        # write a known string, then read until we get it back
        # to clear pipe of spurious data
        puts("clearing the pipe");
        puts "__clear_pipe__"
        loop{
            gotmsg = gets() ;
            # gotmsg = GetDebugInput("") ;
            #//puts("Ruby<:"+gotmsg); redundant
            if (gotmsg[0,14] == "__clear_pipe__"): break; end;
        }#endloop
        puts "Got the clear_pipe";

        ENV['EMACS'] = '1'
        #ENV['COLUMNS'] = '120' if ENV['COLUMNS'].to_i < 120

        # requre the Ruby debugger. Invoke ruby-debug (aka rdebug)
        require modifiedRubyDebugger;
        # dont use stale cached scripts, they might have been updated
        LineCache::cached_files().each {|filename| LineCache::update_cache(filename, false);}

        Debugger.start() if (not Debugger.started?);

     rescue Exception => excptn
        $stdout = @savedStdout;
        $stderr = @savedStderr;
        $stdin = @savedStdin;
        puts(excptn.message);
        excptn.backtrace.each{|line| puts(line);}
     end;#begin/rescue

        $stdout.flush();
        $stderr.flush();
    end # RunSuRDebugR

end #module SuRDebug
## --------------------------------------------------------------------

if (not file_loaded?(__FILE__) )
    require 'SuRDebug/SuRDxcom.dll'
    rc = SuRDebug.RunSuRDebug();
    puts "Loading FileName:#{__FILE__}";
    if (rc == false): return; end;
    ## -----------------------------------------------------------------##
    # If you're seeing this from the debugger, you need to "continue"   ##
    # or "Quit Debugger" then close this file.                          ##
    ##------------------------------------------------------------------##
    UI.start_timer(3,true) {
        print( "\032Debugger idle") if (Debugger.started?); $stdout.flush();
    }
else
    # clear cached files in case they've been edited
    LineCache::cached_files().each {|filename| LineCache::update_cache(filename, false);}
    Debugger.breakpoints.clear
end;
file_loaded(__FILE__)
puts "\032Debugger initialized"


